var path = require('path');
var fs = require('fs');
var fse = require('fs-extra');
var util = require('./util');
var Page = require('./page');
var AppWxss = require('./app_wxss');
var Watchpack = require("watchpack");
var debug = require('debug')('wevue');

var Backend = require('./backend');
var BackendConfig = require('./backend_config');
var BackendWxss = require('./app_admin_wxss');


//加入wath功能，
//如果是wxml，转化后copy到目标目录
//如果是wxss(非app.wxss),转化后copy到目标目录
//其它文件直接copy到目标目录
module.exports = function(folder, vueFolder, rootFolder, wevueFolder, program) {

    debug('wath folder', folder);
    var ignored = new RegExp(program.output);

    var wp = new Watchpack({
        // options:
        aggregateTimeout: 1000,
        // fire "aggregated" event when after a change for 1000ms no additonal change occured
        // aggregated defaults to undefined, which doesn't fire an "aggregated" event

        poll: true,
        // poll: true - use polling with the default interval
        // poll: 10000 - use polling with an interval of 10s
        // poll defaults to undefined, which prefer native watching methods
        // Note: enable polling when watching on a network path

        ignored: ignored,
        // anymatch-compatible definition of files/paths to be ignored
        // see https://github.com/paulmillr/chokidar#path-filtering
    });

    // Watchpack.prototype.watch(string[] files, string[] directories, [number startTime])
    wp.watch([], [folder], Date.now() - 10000);
    // starts watching these files and directories
    // calling this again will override the files and directories

    wp.on("change", function(file, mtime) {
        // filePath: the changed file
        // mtime: last modified time for the changed file

        if (ignored.exec(file)) {
            return;
        }
        console.log(file, 'changed');
        var rPath = file.replace(folder, '');
        debug('rPath', rPath);

        if ((/\.wxml$/.test(file) || /\.wxss$/.test(file))) {
            //生成新的.vue文件
            if (rPath === '/app.wxss' || rPath === '\\app.wxss') {

                debug('app.wxss --> app.css');
                AppWxss(folder, vueFolder, rootFolder);

            } else if (rPath === '/app-admin.wxss' || rPath === '\\app-admin.wxss') {
                //app-admin.wxss to app-admin.css
                debug('app-admin.wxss --> app-admin.css');
                BackendWxss(wxFolder, vueFolder, rootFolder);

            } else if (rPath === '/backend.wxss' || rPath === '\\backend.wxss') {
                //生成backend.vue
                debug('backend.wxss --> backend.vue');
                Backend(folder, vueFolder, rootFolder, wevueFolder, program);

            } else if (rPath === '/backend.wxml' || rPath === '\\backend.wxml') {

                debug('backend.wxml --> backend.vue');
                Backend(folder, vueFolder, rootFolder, wevueFolder, program);

            } else {
                debug('wx page  --> vue page');
                var pathInfo = path.parse(rPath);

                debug(pathInfo.dir);
                debug(pathInfo.name);

                var page = path.join(pathInfo.dir, pathInfo.name);
                Page(folder, vueFolder, page, rootFolder, wevueFolder, function(err, res) {
                    if (err) {
                        console.log('err', err);
                    }
                });
            }

        } else {
            var dest = path.join(vueFolder, 'src', rPath);
            debug('source', file);
            debug('desc', dest);
            var readerStream = fs.createReadStream(file);
            var writerStream = fs.createWriteStream(dest);
            readerStream.pipe(writerStream);
        }
    });
}
