async function handle(params, ctx, next) {

    console.log("==> call /api/collections", params);

    let collectionName = params.collection;
    let collections = await ctx.mongo.collection(collectionName);
    let data = null;
    switch (ctx.request.method) {
        case 'GET':
            if (params.id) {
                data = await collections.findOne({
                    _id: params.id
                });
                ctx.succeed(data);
            } else {
                let query = JSON.parse(params.query);
                data = await collections.find(query);
                ctx.succeed(data);
            }

            break;
        case 'POST':
            data = params.data;
            if (data) {
                data.id = ctx.uuid();
                let result = await collections.insert();
                ctx.succeed(result);
            } else {
                ctx.fail(new Error("数据为空"));
            }
            break;
        case 'PUT':
            if (params.data && params.data._id) {
                data = await collections.save(params.data);
                ctx.succeed(data);
            } else {
                ctx.fail(new Error("_id为空"));
            }
            break;
        case 'DELETE':
            if (params.force) {
                data = await collections.update({
                    _id: params.id
                }, {
                    $set: {
                        deleted: true,
                        deletedAt: new Date()
                    }
                });

            } else {
                data = await collections.remove({
                    _id: params.query.id
                });
            }
            ctx.succeed(data);
            break;
        default:
            ctx.succeed("");
    }
}

module.exports = handle;
