async function handle(params, ctx, next) {
    if(params.entry){
      switch(params.entry){
        case "init":
          ctx.succeed({
            source: await getSource(params, ctx)
          });
        break;
        case "saveSource":
          await saveSource(params, ctx);
        break;
        case "getSource":
          ctx.succeed(await getSource(params, ctx));
        break;
        case "saveSourceData":
          await saveSourceData(params, ctx);
        break;
        case "getSourceData":
          await getSourceData(params, ctx);
        break;
        case "getSourceData_Table":
          await getSourceData_Table(params, ctx);
        break;
        default:
          ctx.fail(new Error("switch default"));
      }
    }else{
      ctx.fail(new Error("entry not defined"));
    }
}
async function getSource(params, ctx){
  try{
    var coll = await ctx.mongo.collection("source");
    var data = JSON.parse(params.data || "{}");
    console.log(data);
    var where = {};
    if(data._id){
      where._id = data._id;
      return await coll.findOne(where);
    }else{
      return await coll.find(where).toArray();
    }
  }catch(e){
    console.log(e);
    return {};
  }
}
async function saveSource(params, ctx) {
  console.log(params);
  try{
    var coll = await ctx.mongo.collection("source");
    var logsColl = await ctx.mongo.collection("source_log");

    coll.ensureIndex({name: 1}, { unique: true });
    var data = JSON.parse(params.data);
    if(data.deleted){
      await coll.remove({
        _id: data._id
      });
      var delColl = await ctx.mongo.collection("DS_"+data._id);
      delColl.drop();
      ctx.succeed("数据源删除成功！");
      return;
    }
    var res = await coll.update({
      _id: data._id || ctx.uuid()
    },{
      $set: data
    },{
      upsert: true
    });
    if(res.result && res.result.n === 1){
      await logsColl.insert({
        source: data,
        type: data.deleted?"deleted":"updated",
        updatedAt: new Date().getTime(),
        updatedBy: "user name"
      });
      ctx.succeed("数据更改成功！");
    }else{
      ctx.fail(new Error("保存更改失败！"));
    }
  }catch(e){
    ctx.fail(new Error(JSON.stringify(e)));
  }
}

async function saveSourceData(params, ctx){
  console.log(params);
  try{
    var res = {};
    var data = JSON.parse(params.data);
    var coll = await ctx.mongo.collection("DS_"+data.sourceid);
    var source = await getSource({data:JSON.stringify({_id: data.sourceid})}, ctx);
    console.log(source);
    if(data.deleted){
      data.deletedAt = new Date().getTime();
    }
    source.fields.some(function(f){
      if(f.type === "string_unique"){
        var index = {};
        index[f.name] = 1;
        coll.ensureIndex(index, { unique: true });
      }
    });
    res = await coll.update({
      _id: data._id || ctx.uuid()
    }, {
      $set: data
    },{
      upsert: true
    });
    if(res.result && res.result.n === 1){
      ctx.succeed("数据更改成功！");
    }else{
      ctx.fail(new Error("保存更改失败！"));
    }
  }catch(e){
    ctx.fail(new Error(JSON.stringify(e)));
  }
}
async function getSourceData_Table(params, ctx){
  var data = JSON.parse(params.data || "{}");
  var coll = await ctx.mongo.collection("DS_"+ data.sourceid);
  var where = {
      deleted: {
        $ne: true
      }
    };
    var sortCols = [];
    if(data.fields.length>0){
      data.fields.some(function(f){
        if(f.sTitle != "操作"){
          sortCols.push(f.sTitle);
        }
      });
    }
    var options = {
      'sort': [
        [sortCols[parseInt(params.iSortCol_0)], params.sSortDir_0]
      ],
      'limit': parseInt(params.iDisplayLength),
      'skip': parseInt(params.iDisplayStart)
    };
    var totalCount = await coll.count(where);
    console.log('总数: ' + totalCount);

    if (params.sSearch) {
      var search = [];
      var search_text = new RegExp('\S*' + params.sSearch + '\S*');
      if(sortCols.length>0){
        sortCols.forEach(function(s){
          let _s = {};
          _s[s] = search_text;
          search.push(_s);
        });
      }
      where.$or = search;
    }

    var displayCount = await coll.count(where);
    console.log('搜索结果数: ' + displayCount);

    var data = await coll.find(where, {}, options).toArray();

    var result = {
      "sEcho": params.sEcho,
      "iTotalRecords": totalCount,
      "iTotalDisplayRecords": displayCount,
      "aaData": []
    };
    if(data.length>0){
      data.some(function(item){
        var rowData = {};
        sortCols.some(function(name, index){
          rowData[index] = item[name]||"";
        });
        rowData[sortCols.length] = '<a style="margin-right: 15px;" data-id="'+item._id+'" class="edit_source_data" data-type="edit">编辑</a>'+
                                   '<a data-id="'+item._id+'" class="edit_source_data" data-type="remove">删除</a>';
        result.aaData.push(rowData);
      });
    }
    ctx.succeed(result);
}
async function getSourceData(params, ctx){
  try{
    var data = JSON.parse(params.data || "{}");
    var coll = await ctx.mongo.collection("DS_"+data.sourceid);
    console.log(data);
    var where = {};
    if(data._id){
      where._id = data._id;
      ctx.succeed(await coll.findOne(where));
    }else{
      ctx.succeed(await coll.find(where).toArray());
    }
  }catch(e){
    ctx.fail(new Error(JSON.stringify(e)));
  }
};

module.exports = handle;
